@@ -8,8 +8,10 @@ import logging
 import hashlib
 import socket
 import re
+import fnmatch
 import mimetypes
 import calendar
+import functools
 from contextlib import closing  # for Python2.6 compatibility
 from gzip import GzipFile
 
@@ -20,12 +22,12 @@ import grandfatherson
 from byteformat import ByteFormatter
 
 from bakthat.backends import GlacierBackend, S3Backend, RotationConfig, SwiftBackend
-from bakthat.conf import config, load_config, DEFAULT_DESTINATION, DEFAULT_LOCATION, CONFIG_FILE
+from bakthat.conf import config, load_config, DEFAULT_DESTINATION, DEFAULT_LOCATION, CONFIG_FILE, EXCLUDE_FILES
 from bakthat.utils import _interval_string_to_seconds
 from bakthat.models import Backups
 from bakthat.sync import BakSyncer, bakmanager_hook, bakmanager_periodic_backups
 
-__version__ = "0.5.4"
+__version__ = "0.5.5"
 
 app = aaargh.App(description="Compress, encrypt and upload files directly to Amazon S3/Glacier/Swift.")
 
@@ -169,6 +171,27 @@ def rotate_backups(filename, destination=None, profile="default", config=CONFIG_
     return deleted
 
 
+def _get_exclude(exclude_file):
+    """ Load a .gitignore like file to exclude files/dir from backups.
+
+    :type exclude_file: str
+    :param exclude_file: Path to the exclude file
+
+    :rtype: function
+    :return: A function ready to inject in tar.add(exlude=_exclude)
+
+    """
+    patterns = filter(None, open(exclude_file).read().split("\n"))
+
+    def _exclude(filename):
+        for pattern in patterns:
+            if re.search(fnmatch.translate(pattern), filename):
+                log.debug("{0} excluded".format(filename))
+                return True
+        return False
+    return _exclude
+
+
 @app.cmd(help="Backup a file or a directory, backup the current directory if no arg is provided.")
 @app.cmd_arg('filename', type=str, default=os.getcwd(), nargs="?")
 @app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
@@ -177,7 +200,9 @@ def rotate_backups(filename, destination=None, profile="default", config=CONFIG_
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
 @app.cmd_arg('-c', '--config', type=str, default=CONFIG_FILE, help="path to config file")
 @app.cmd_arg('-k', '--key', type=str, default=None, help="Custom key for periodic backups (works only with BakManager.io hook.)")
-def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profile="default", config=CONFIG_FILE, key=None, **kwargs):
+@app.cmd_arg('--exclude-file', type=str, default=None)
+@app.cmd_arg('--s3-reduced-redundancy', action="store_true")
+def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profile="default", config=CONFIG_FILE, key=None, exclude_file=None, s3_reduced_redundancy=False, **kwargs):
     """Perform backup.
 
     :type filename: str
@@ -220,6 +245,19 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
         backup_file_fmt = "{0}.{1}"
 
     log.info("Backing up " + filename)
+
+    if exclude_file and os.path.isfile(exclude_file):
+        EXCLUDE_FILES.insert(0, exclude_file)
+
+    _exclude = lambda filename: False
+    if os.path.isdir(filename):
+        join = functools.partial(os.path.join, filename)
+        for efile in EXCLUDE_FILES:
+            efile = join(efile)
+            if os.path.isfile(efile):
+                _exclude = _get_exclude(efile)
+                log.info("Using {0} to exclude files.".format(efile))
+
     arcname = filename.strip('/').split('/')[-1]
     now = datetime.utcnow()
     date_component = now.strftime("%Y%m%d%H%M%S")
@@ -267,9 +305,10 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     else:
         # If not we compress it
         log.info("Compressing...")
+
         with tempfile.NamedTemporaryFile(delete=False) as out:
             with closing(tarfile.open(fileobj=out, mode="w:gz")) as tar:
-                tar.add(filename, arcname=arcname)
+                tar.add(filename, arcname=arcname, exclude=_exclude)
             outname = out.name
             out.seek(0)
             backup_data["size"] = os.fstat(out.fileno()).st_size
@@ -307,7 +346,7 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     backup_data["backend_hash"] = hashlib.sha512(access_key + container_key).hexdigest()
 
     log.info("Uploading...")
-    storage_backend.upload(stored_filename, outname)
+    storage_backend.upload(stored_filename, outname, s3_reduced_redundancy=s3_reduced_redundancy)
 
     # We only remove the file if the archive is created by bakthat
     if bakthat_compression or bakthat_encryption:
