@@ -25,6 +25,10 @@ class BaseModel(peewee.Model):
     class Meta:
         database = database
 
+    @classmethod
+    def create(cls, **attributes):
+        return super(BaseModel, cls).create(**attributes)
+
 
 class Backups(BaseModel):
     """Backups Model."""
